from __future__ import division
__author__ = 'pav'
import os, exifread, subprocess
from shutil import rmtree
from datetime import datetime
import platform

if platform.system()== 'Windows':
    EXTERNAL_TOOLS = {
        "ffprobe": r"C:\ffmpeg\bin\ffprobe.exe",
        "ffmpeg": r"C:\ffmpeg\bin\ffprobe.exe",
        "ffplay": r"C:\ffmpeg\bin\ffplay.exe"
    }

    REST_CALL = {
        'region':"http://127.0.0.1:8000/get_region/",
        'image': "http://127.0.0.1:8000/get_last/image/"
    }
else:
    EXTERNAL_TOOLS = {
        "ffprobe": r"ffprobe",
        "ffmpeg": r"ffprobe",
        "ffplay": r"ffplay"
    }
    REST_CALL = {
        'region':"http://95.85.57.196/get_region/",
        'image': "http://95.85.57.196/get_last/image/"
    }






def MinuteToSec(duration):
    h, m, sec = duration.split(":")
    return int(m)*60 + int(float(sec))

def UpdateListColumn(data, index, offset):
    """
        Add an offfset to a columns list of a list
    """
    for i in data: i[index] = int(i[index]) + offset
    return data


def GetDuration(video):
    """
        Returns the duration of a video in seconds
    """
    result = subprocess.Popen([EXTERNAL_TOOLS["ffprobe"], video],
    stdout = subprocess.PIPE, stderr = subprocess.STDOUT)
    log =  [x for x in result.stdout.readlines() if "Duration" in x]
    a = log[0].split(",")[0].split(" ")[3]
    a = MinuteToSec(a)
    return a


def MetaReader(img):
    """
        Returns lat, lon, timestamp, shooting angle given a photo
    """
    f = open(img, 'rb')
    tags = exifread.process_file(f)

    try:
        angle = tags['GPS GPSImgDirection'].values.pop()
        tlon = tags['GPS GPSLongitude'].values
        tlat = tags['GPS GPSLatitude'].values
        h, m, s =  tags["GPS GPSTimeStamp"].values

        lon = tlon[0].num/tlon[0].den + tlon[1].num/tlon[1].den/60.0 + tlon[2].num/tlon[2].den/3600.0
        lat = tlat[0].num/tlat[0].den + tlat[1].num/tlat[1].den/60.0 + tlat[2].num/tlat[2].den/3600.0

        hour = int(h.num/h.den)
        sec = int(s.num/s.den)
        min = int(m.num/m.den)
        date = "-".join(tags["GPS GPSDate"].values.split(":"))

        datum = "{} {}:{}:{}".format(date,str(h), str(min),str(sec))
        #try:
        timestamp = datetime.strptime(datum, '%Y-%m-%d %I:%M:%S')

        #except ValueError:
        #    print dir(h)
        #    print h
        #    print min, sec

        return lat, lon, timestamp, angle
    except KeyError:
        return False

    except ValueError:
        return False

    finally:
        f.close()


class FileSystem():
    """
        class for file/dirs handling functionality
    """

    def __init__(self, p):
        self.path = p

    def __repr__(self):
        return """
        Filesystem Object:
            root dir: {}
            subdirs: {}
        """.format(self.path, str(os.listdir(self.path)))

    @staticmethod
    def getfiles(p):
        return [os.path.join(p, i) for i in os.listdir(p) if i != 'Thumbs.db']

    @staticmethod
    def getdirs(p):
        return { i: os.path.join(p, i) for i in os.listdir(p) \
                 if os.path.isdir(os.path.join(p, i)) }

    @staticmethod
    def mkdir(p):
        if not os.path.exists(p): os.mkdir(p)

    @staticmethod
    def checkdir(p):
        return os.path.isdir(p)

    @staticmethod
    def checkfile(f):
        return os.path.isfile(f)

    @staticmethod
    def serverpath(p):
        return '/'.join( ['photos'] + [p.replace('\\','/').split("/")[-3], p.replace('\\','/').split("/")[-1]])

    @staticmethod
    def videopath(p):
        return '/'.join( ['photos'] + p.replace("\\", '/').split("/")[-3:] )

    @staticmethod
    def remove(p):
        try:
            return rmtree(p, ignore_errors=False, onerror=None)
        except:
            return True

    @classmethod
    def getdata(cls, p):
        if cls.checkdir(p):
            return cls.getfiles(os.path.join(p, "Wide")), cls.getfiles(os.path.join(p, 'Tele'))\
                ,cls.getfiles(os.path.join(p, "video"))
        else:
            return False






if __name__ == '__main__':
    pass
    #print FileSystem.remove("C:\Users\pav\Desktop\moufa")
    #print FileSystem.getdata("C:\Users\pav\Desktop\Saronikos_8_Poros")
    #a = FileSystem.getdirs("C:\Users\pav\Desktop\Saronikos_8_Poros")
    #print a
    #print FileSystem.getfiles(a["Wide"])
    #C:\\Users\\pav\\Desktop\\Saronikos_8_Poros\\Tele\\_TIP1688.jpg