__author__ = 'pav'
import requests

from dataset.core.utils import MetaReader, GetDuration, REST_CALL
from dataset.core.base import DataSet


class ImageDataset(DataSet):
    """
        Image Dataset Class subclassing Dataset Interface
    """

    def __init__(self, dir):
        self._dirs = super(ImageDataset, self).setdir(dir)
        self.__data = None

    @property
    def getimageID(self):
        r = requests.get(REST_CALL["image"])
        if r.json()["last_id"] == None:
            return 1
        else:
            return r.json()["last_id"] + 1

    @property
    def getregionID(self):
        r = requests.get(REST_CALL["region"])
        return r.json()["last_id"]

    def _gps_check(self, mode):
        return len(self.__data[mode]) == len(set([(i[1], i[2]) for i in self.__data[mode]]))

    def extract(self):
         self.__data = {
            "Tele": [
                [self.djangopath(i)] + list(MetaReader(i)) for i in \
                self.getdatafiles(self._dirs["Tele"]) if MetaReader(i)!= False
            ],
            "Wide": [
                [self.djangopath(i)] + list(MetaReader(i)) for i in \
                self.getdatafiles(self._dirs["Wide"]) if MetaReader(i) != False
            ],
            "video": [
                [ self.servervideo(i), GetDuration(i)] for i in self.getdatafiles(self._dirs["video"])
            ]
        }

    def process(self):
        #--- Ordering ------

        for i in self.__data["Wide"]: i[3] = self.epoch(i[3])
        for i in self.__data["Tele"]: i[3] = self.epoch(i[3])

        self.__data["Wide"] = self.datasort(self.__data["Wide"], 3)
        self.__data["Tele"] = self.datasort(self.__data["Tele"], 3)

        #------ GPS fix -----
        while not self._gps_check("Wide"):
            self.__data["Wide"] = self.gpsfix(self.__data["Wide"])

        while not self._gps_check("Tele"):
            self.__data["Tele"] = self.gpsfix(self.__data["Tele"])

        #--- UUID's ---------
        self._uuid()

        #--------- Circle Nav -------
        self._circlenav(self.__data["Wide"])
        self._circlenav(self.__data["Tele"])

        #------ time to play ----

        self._playtime()
        #self.video_check()
        self._videotag()

        #---------- Region id ------
        region_pk = self.getregionID

        for i in self.__data["Wide"]: i.append(region_pk)
        for i in self.__data["Tele"]: i.append(region_pk)

    def gpsfix(self, data):
        gps_all = [(i[1], i[2]) for i in data]

        for i in set(gps_all):
            gps_all.remove(i)

        gps_tmp = {(i[1], i[2]): data.index(i) for i in data}

        for i in gps_all:
            data[gps_tmp[i]][1], data[gps_tmp[i]][2] = self._offset(i)
        return data

    def _offset(self, xy):
        return [str(float(i) + float(0.00001)) for i in xy]

    def _playtime(self):
        start_playtime = self.__data["Wide"][0][3]

        for i in self.__data["Wide"]:
            i.append( i[3] - start_playtime )

        start_playtime = self.__data["Tele"][0][3]
        offset = abs(self.__data["Tele"][0][3] -  self.__data["Wide"][0][3])

        for i in self.__data["Tele"]:
            i.append( i[3] - start_playtime + offset)

    def video_check(self):
        a = [ i[-1] for i in self.__data["Wide"]]
        b = [ i[-1] for i in self.__data["Tele"]]
        print "----- Time to play ---------"
        print len(a+b)
        print len(set(a+b))

    def _videotag(self):
        if len(self.__data["video"]) > 1:
            for video in self.__data["video"]:
                for i in self.__data["Wide"]:
                    if i[-1] >= video[1]:
                        i[-1] -= video[1]
                        i.append(video[0])
                for i in self.__data["Tele"]:
                    if i[-1] >= video[1]:
                        i[-1] -= video[1]
                        i.append(video[0])
        else:
            for i in self.__data["Wide"]:
                i.append(self.__data["video"][0][0])
            for i in self.__data["Tele"]:
                i.append(self.__data["video"][0][0])



    def _uuid(self):
        start_id = self.getimageID
        for i in self.__data["Wide"]:
            i.append(start_id)
            start_id += 1
        for i in self.__data["Tele"]:
            i.append(start_id)
            start_id +=1

    def _circlenav(self, data):
        for i in data:
            i.append(data[data.index(i)-1][5])
            if data.index(i) == len(data) - 1:
                i.append(data[0][5])
            else:
                i.append(data[data.index(i)+1][5])

    def refresh(self, dir):
        self._dirs = super(ImageDataset, self).setdir(dir)

    def export(self):
        return self.__data









if __name__ == '__main__':
    a = ImageDataset("C:\Users\pav\Desktop\Saronikos_8_Poros")
    print a.getregionID
